%==========================================================================
% This script produces the results for Table 1 in the Federal Reserve Bank
% of Cleveland commentary "Using Advance Layoff Notices as a Labor Market 
% Indicator" by Pawel M. Krolikowski, Kurt G. Lunsford and Meifeng Yang. 
% 
% This script calls data from Data.xlsx.
% Date last modified: 12/18/2019 
%==========================================================================

clc
clear;
format short g
tic;

%==========================================================================
% Parameters of the Script
%==========================================================================
p = 3;              %number of lags in the regression
NWtrunc = 4;        %truncation parameter for Newey-West

index = 2;          %choose 1 for UI claims
                    %choose 2 for changes in unemployment rate
                    %choose 3 for changes in employment

%==========================================================================
% Load the Data
%==========================================================================
Data = xlsread('Data.xlsx','Sheet1','B2:F283');
F = Data(:,4);      %the WARN factor
MSE = Data(:,5);    %the MSE of the WARN factor

T = size(Data,1);   %number of observations

%==========================================================================
% Organize the Data
%==========================================================================
%construct the left-hand side variables
if index == 1               %UI claims
    h = 1;
    LHS = Data(3:T,1);
    LHS = LHS(p+h:T-2,1);
elseif index == 2           %unemployment rate changes
    h = 3;
    LHS = (Data(3:T,2) + Data(2:T-1,2) + Data(1:T-2,2))/3;
    LHS = LHS(p+h:T-2,1);
elseif index == 3           %private employment changes
    h = 3;
    LHS = 1000*(Data(3:T,3) + Data(2:T-1,3) + Data(1:T-2,3))/3;
    LHS = LHS(p+h:T-2,1);
else
    disp('Error. "index" must takes values equal 1, 2 or 3')
    return
end

%construct the right-hand side variables
RHS = ones(T-p-1-h,1);
for jj = 1:p
    RHS = [RHS,Data(p+3-jj:T-h+1-jj,1:3)];
end
for jj = 1:p
    RHS = [RHS,F(p+3-jj:T-h+1-jj,1)];
end

%dimensions ofthe right-hand-side data
[TT,NN] = size(RHS);

%==========================================================================
% Least Squares Regression
%==========================================================================
Beta = (RHS'*RHS)\(RHS'*LHS);       %least squares
V = LHS - RHS*Beta;                 %regression errors

%==========================================================================
% Estimate Omega_v with Newey and West (1987)
%==========================================================================
%bartlett kernel weights
k = 1 - (1:NWtrunc-1)'/NWtrunc;

%compute Omega_v
z = RHS.*(V*ones(1,NN));
Omega_v = (z'*z)/TT;
for jj = 1:NWtrunc-1
    Gamma = (z(1+jj:TT,:)'*z(1:TT-jj,:))/TT;        %autocovariances
    Omega_v = Omega_v + k(jj,1)*(Gamma + Gamma');   %long-run variance
end

%==========================================================================
% Estimate Omega_f as in the Appendix
%==========================================================================
%compute the autocovariances as on page 4 of the Appendix
Gamma = zeros(NN,NN,p);
for mm = 1:p
    D = zeros(TT-mm+1,1);
    for jj = 1:p-mm+1
        D = D + Beta(NN-p+jj,1)*Beta(NN-p+jj+mm-1,1)*...
            MSE(p+3-jj:T-h+1-jj-mm+1,1);
    end
    D = diag(D);
    Gamma(:,:,mm) = RHS(mm:TT,:)'*D*RHS(1:TT-mm+1,:)/TT;
end

%compute Omega_v with Bartlett kernel weights
Omega_f = Gamma(:,:,1);
for jj = 1:min(p-1,NWtrunc-1)
    Omega_f = Omega_f + k(jj,1)*(Gamma(:,:,jj+1) + Gamma(:,:,jj+1)');
end

%==========================================================================
% Standard Errors and Wald Statistics
%==========================================================================
%the covariance matrix
Q = (RHS'*RHS)/TT;
Omega = Omega_v + Omega_f;
VV = Q\(Omega/Q);

%standard errors
se = sqrt(diag(VV)/(TT - NN));     %divide by TT-NN to match Stata

%collect the results for Table 1
Results = [Beta(NN-p+1:NN,1),se(NN-p+1:NN,1)];

%wald statistic and p-value
Wald = (TT - NN)*Beta(NN-p+1:NN,1)'*...
    (VV(NN-p+1:NN,NN-p+1:NN)\Beta(NN-p+1:NN,1));    %multiply by TT-NN to match Stata
p = 1 - chi2cdf(Wald,p);


toc;

